package yonyou.udh.demo;

import org.apache.spark.api.java.*;
import org.apache.spark.api.java.function.Function;
import org.apache.spark.mllib.linalg.distributed.RowMatrix;
import org.apache.spark.mllib.linalg.Matrix;
import org.apache.spark.mllib.linalg.SingularValueDecomposition;
import org.apache.spark.mllib.linalg.Vector;
import org.apache.spark.mllib.linalg.Vectors;
import org.apache.spark.SparkConf;

public class SVD {
  public static void main(String[] args) {
    SparkConf conf = new SparkConf().setAppName("SVD Example");
    JavaSparkContext sc = new JavaSparkContext(conf);
    String path = "hdfs://udh-spark-01:8020/tmp/mllib/dr.txt";
    JavaRDD<String> data = sc.textFile(path);
    
    JavaRDD<Vector> parsedData = data.map(
    	      new Function<String, Vector>() {
    	        public Vector call(String s) {
    	          String[] sarray = s.split(" ");
    	          double[] values = new double[sarray.length];
    	          for (int i = 0; i < sarray.length; i++)
    	            values[i] = Double.parseDouble(sarray[i]);
    	          return Vectors.dense(values);
    	        }
    	      }
    );
   
    // Create a RowMatrix from JavaRDD<Vector>.
    RowMatrix mat = new RowMatrix(parsedData.rdd());
    // Compute the top 4 singular values and corresponding singular vectors.
    SingularValueDecomposition<RowMatrix, Matrix> svd = mat.computeSVD(3, true, 1.0E-9d);
    RowMatrix U = svd.U();
    Vector s = svd.s();
    Matrix V = svd.V();
    System.out.println("---------------------begin--------------------------");
    System.out.println("RowMatrix U = " + U+" NUM ROWS="+U.numRows()+" NUM COLS="+U.numCols());
    System.out.println("Vector s = " + s);
    System.out.println("Matrix V = " + V);
    System.out.println("Matrix V NUM ROWS= " + V.numRows()+" NUM COLS="+V.numCols());
    System.out.println("---------------------end--------------------------");
    sc.stop();
    
  }
}